//
//	Neatpad - simple text editor application
//
//	www.catch22.net
//	Written by J Brown 2004
//
//	Freeware
//

import win32.core;
import win32.commdlg;
import std.c.stdlib;
import textview.textview;
import resource;

const char[] APP_TITLE   = "Neatpad";
const char[] WEBSITE_STR = "www.catch22.net";

TCHAR[]		szAppName = APP_TITLE;
HWND		hwndMain;
HWND		hwndTextView;

TCHAR[MAX_PATH] szFileName;
TCHAR[MAX_PATH] szFileTitle;



BOOL ShowOpenFileDlg(HWND hwnd, PSTR pstrFileName, PSTR pstrTitleName)
{
	TCHAR[] szFilter = "Text Files (*.txt)\0*.txt\0All Files (*.*)\0*.*\0\0";
	
	OPENFILENAME ofn; // = { ofn.sizeof };

	ofn.hwndOwner		= hwnd;
	ofn.hInstance		= GetModuleHandle(null);
	ofn.lpstrFilter		= szFilter;
	ofn.lpstrFile		= pstrFileName;
	ofn.lpstrFileTitle	= pstrTitleName;
	
	ofn.nFilterIndex	= 1;
	ofn.nMaxFile		= _MAX_PATH;
	ofn.nMaxFileTitle	= _MAX_FNAME + _MAX_EXT; 

	// flags to control appearance of open-file dialog
	ofn.Flags			=	OFN_EXPLORER			| 
							OFN_ENABLESIZING		|
							OFN_ALLOWMULTISELECT	| 
							OFN_FILEMUSTEXIST;

	return GetOpenFileName(&ofn);
}

void ShowAboutDlg(HWND hwndParent)
{
	MessageBox( hwndParent, 
				APP_TITLE ~ "\r\n\r\n" ~ WEBSITE_STR, 
				APP_TITLE, 
				MB_OK | MB_ICONINFORMATION
				);
}

void SetWindowFileName(HWND hwnd, TCHAR[] szFileName)
{
	SetWindowText(hwnd, szFileName ~ " - " ~ szAppName);
}

//
//	Main window procedure
//
static extern (Windows)
LRESULT WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	static int width, height;

	switch (msg)
	{
	case WM_CREATE:
		hwndTextView = CreateTextView(hwnd);

		// automatically create new document when we start
		PostMessage(hwnd, WM_COMMAND, IDM_FILE_NEW, 0);
		return 0;

	case WM_DESTROY:
		PostQuitMessage(0);
		return 0;

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDM_FILE_NEW:
			SetWindowFileName(hwnd, "Untitled");
			return 0;

		case IDM_FILE_OPEN:

			// get a filename to open
			if (ShowOpenFileDlg(hwnd, szFileName, szFileTitle))
			{
				SetWindowFileName(hwnd, szFileTitle);
			}

			return 0;

		case IDM_HELP_ABOUT:
			ShowAboutDlg(hwnd);
			return 0;

		default:
			return 0;
		}

	case WM_CLOSE:
		DestroyWindow(hwnd);
		return 0;

	case WM_SIZE:
		width  = cast(short)LOWORD(lParam);
		height = cast(short)HIWORD(lParam);

		MoveWindow(hwndTextView, 0, 0, width, height, TRUE);
		return 0;

	default:
		return DefWindowProc(hwnd, msg, wParam, lParam);
	}
}

//
//	Register main window class
//
void InitMainWnd()
{
	WNDCLASSEX wcx;
	HANDLE hInst = GetModuleHandle(null);

	// Window class for the main application parent window
	wcx.cbSize			= wcx.sizeof;
	wcx.style			= 0;
	wcx.lpfnWndProc		= &WndProc;
	wcx.cbClsExtra		= 0;
	wcx.cbWndExtra		= 0;
	wcx.hInstance		= hInst;
	wcx.hCursor			= LoadCursor (null, IDC_ARROW);
	wcx.hbrBackground	= cast(HBRUSH)0;
	wcx.lpszMenuName	= MAKEINTRESOURCE(IDR_MENU1);
	wcx.lpszClassName	= szAppName;
	wcx.hIcon			= LoadImage(hInst, MAKEINTRESOURCE(IDI_ICON1), IMAGE_ICON, 32, 32, LR_CREATEDIBSECTION);
	wcx.hIconSm			= LoadImage(hInst, MAKEINTRESOURCE(IDI_ICON1), IMAGE_ICON, 16, 16, LR_CREATEDIBSECTION);

	RegisterClassEx(&wcx);
}

//
//	Create a top-level window
//
HWND CreateMainWnd()
{
	return CreateWindowEx(0,
				szAppName,				// window class name
				szAppName,				// window caption
				WS_OVERLAPPEDWINDOW,//|WS_CLIPCHILDREN,
				CW_USEDEFAULT,			// initial x position
				CW_USEDEFAULT,			// initial y position
				560,					// initial x size
				320,					// initial y size
				null,					// parent window handle
				null,					// use window class menu
				GetModuleHandle(null),	// program instance handle
				null);					// creation parameters
}

//
//	Entry-point for text-editor application
//
extern (Windows)
int main()
{
	MSG			msg;
	HACCEL		hAccel;

	// initialize window classes
	InitMainWnd();
	InitTextView();

	// create the main window!
	hwndMain = CreateMainWnd();

	STARTUPINFO info;
	GetStartupInfo(&info);

	ShowWindow(hwndMain, info.wShowWindow);

	// load keyboard accelerator table
	hAccel = LoadAccelerators(GetModuleHandle(null), MAKEINTRESOURCE(IDR_ACCELERATOR1));

	// message-loop
	while( GetMessage(&msg, null, 0, 0) > 0)
	{
		if (!TranslateAccelerator(hwndMain, hAccel, &msg))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	return 0;
}
