//
//	MODULE:		TextView.cpp
//
//	PURPOSE:	Implementation of the TextView control
//
//	NOTES:		www.catch22.net
//

module textview.textview;

import win32.core;

// internal
class TextView
{
	this(HWND hwnd) { m_hWnd = hwnd; }
	HWND	m_hWnd;
	int x;
	static TextView[HWND] m_windows;

	//
	//	Painting procedure for TextView objects
	//
	extern (Windows)
	LRESULT OnPaint()
	{
		HDC			hdc;
		PAINTSTRUCT ps;
		RECT		rect;
		char[]		text = "Hello World!";

		HANDLE		hOldFont;
		HFONT		hFont;

		hdc = BeginPaint(m_hWnd, &ps);

		hFont = cast(HFONT)GetStockObject(ANSI_FIXED_FONT);
		hOldFont = SelectObject(hdc, hFont);

		GetClientRect(m_hWnd, &rect);

		ExtTextOut(hdc, 10, 10, ETO_OPAQUE, &rect, text, text.length, null);

		SelectObject(hdc, hOldFont);

		EndPaint(m_hWnd, &ps);

		return 0;
	}

	//
	//	Win32 TextView window procedure.
	//
	static extern (Windows)
	LRESULT TextViewWndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
	{
		TextView* ptv = (hwnd in m_windows);

		switch (msg)
		{
		// First message received by any window - make a new TextView object
		// and store pointer to it in our extra-window-bytes
		case WM_NCCREATE:
			m_windows[hwnd] = new TextView(hwnd);
			return TRUE;

		// Last message received by any window - delete the TextView object
		case WM_NCDESTROY:
			delete ptv;
			return 0;

		// Draw contents of TextView whenever window needs updating
		case WM_PAINT:
			return ptv.OnPaint();

		default:
			break;
		}

		return DefWindowProc(hwnd, msg, wParam, lParam);
	}
}



const TCHAR[] TEXTVIEW_CLASS = "TextView32";

//
//	TextView API declared here
//

//
//	Register the TextView window class
//
BOOL InitTextView()
{
	WNDCLASSEX	wcx;

	//Window class for the main application parent window
	wcx.cbSize			= wcx.sizeof;
	wcx.style			= CS_PARENTDC;
	wcx.lpfnWndProc		= &TextView.TextViewWndProc;
	wcx.cbClsExtra		= 0;
	wcx.cbWndExtra		= (TextView*).sizeof;
	wcx.hInstance		= GetModuleHandle(null);
	wcx.hIcon			= null;
	wcx.hCursor			= LoadCursor (null, IDC_IBEAM);
	wcx.hbrBackground	= cast(HBRUSH)0;		//NO FLICKERING FOR US!!
	wcx.lpszMenuName	= null;
	wcx.lpszClassName	= TEXTVIEW_CLASS;	
	wcx.hIconSm			= null;

	return RegisterClassEx(&wcx) ? TRUE : FALSE;
}

//
//	Create a TextView control!
//
HWND CreateTextView(HWND hwndParent)
{
	return CreateWindowEx(WS_EX_CLIENTEDGE, 
		TEXTVIEW_CLASS, "", 
		WS_VSCROLL |WS_HSCROLL | WS_CHILD | WS_VISIBLE,
		0, 0, 0, 0, 
		hwndParent, 
		null, 
		GetModuleHandle(null), 
		null);
}

